<?php

/*
	editidentity.inc.php - for modifying and deleting identities in the DB
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/identity.inc.php');

require_once(RESOURCE_DIR . 'includes/textfuzzy.inc.php');

class editidentity
{
	function editidentity(&$db)
	{
		$this->db = &$db;
		$this->prefix = $db->get_prefix();
	}

	function createidentity()
		// creates a new, empty identity in the database and returns a new 
		// identity object with its ID
	{
		$now = TIMENOW;

		$result = $this->db->query("
			INSERT INTO {$this->prefix}identity
			SET
				identity_createddate=$now,
				identity_lastmodifieddate=$now
				");

		list($id) = $this->db->query_single("SELECT LAST_INSERT_ID()");
		if (!$id) trigger_error("Could not insert new identity into database", E_USER_ERROR);

		$identity = new identity();
		$identity->id = $id;
		$identity->createddate = $now;
		$identity->lastmodifieddate = $now;

		return $identity;
	}

	function deleteidentity($identity, $resolution = 'deleted', $refid = NULL)
		// deletes the given identity in the database.  $id is the ID number
	{
		$id = (int)$identity->id;
		$result = $this->db->query("
			DELETE FROM {$this->prefix}identity
			WHERE identity_ID=$id");
			// if porting to another DB, keep in mind that we don't want to ever re-use
			// the same ID value as one that has been deleted in the past.  MySQL's
			// autoincrement should do this

		if (!$result || !$this->db->affected_rows()) return false;

		$now = TIMENOW;
		if (!in_array($resolution, array('deleted', 'merged')))
			$resolution = 'deleted';
		$refid = $resolution == 'merged' ? (int)$refid : 'NULL';

		if ($identity->affiliation != 'none')
		{
			$this->db->query("REPLACE INTO {$this->prefix}identity_alias
				SET identity_alias_ID=$id, identity_alias_resolution='$resolution',
				identity_alias_refID=$refid,
				identity_alias_createddate=$now, identity_alias_lastmodifieddate=$now");
		}

		if ($resolution == 'merged')
		{
			$this->db->query("UPDATE {$this->prefix}identity_alias
			SET identity_alias_refID=$refid, identity_alias_lastmodifieddate=$now
		 	WHERE identity_alias_refID=$id");
			$this->db->query("UPDATE {$this->prefix}person
				SET person_secondidentityID=$refid WHERE person_secondidentityID=$id");
			$this->db->query("UPDATE {$this->prefix}person
				SET person_identityID=$refid WHERE person_identityID=$id");
			$this->db->query("DELETE FROM {$this->prefix}person
				WHERE person_secondidentityID=$refid && person_identityID=$refid");
		}
		elseif ($resolution == 'deleted')
			$this->db->query("UPDATE {$this->prefix}identity_alias
			SET identity_alias_resolution='deleted', identity_alias_refID=NULL,
			identity_alias_lastmodifieddate=$now
			WHERE identity_alias_refID=$id");

		$this->db->query("DELETE FROM {$this->prefix}classification WHERE classification_identityID=$id");
		$this->db->query("DELETE FROM {$this->prefix}event          WHERE event_identityID=$id");
		$this->db->query("DELETE FROM {$this->prefix}identifier     WHERE identifier_identityID=$id");
		$this->db->query("DELETE FROM {$this->prefix}name           WHERE name_identityID=$id");
		$this->db->query("DELETE FROM {$this->prefix}organisation   WHERE organisation_identityID=$id");
		$this->db->query("DELETE FROM {$this->prefix}person         WHERE person_identityID=$id");
		$this->db->query("DELETE FROM {$this->prefix}person         WHERE person_secondidentityID=$id");
		$this->db->query("DELETE FROM {$this->prefix}resource       WHERE resource_identityID=$id");

		$controls = &controls::getinstance();
		$indexer = $controls->getcontrol('indexer');
		$indexer->deletefromindex($identity);

		return true;
	}

	function getunverifiedstatus($obj)
	{
		if (!$obj->verified) $this->hasunverified = true;
	}


	function updateidentity($identity)
		// updates the identity in the database, specified by the ID number given,
		// with the given identity object.  The data in the database will be synced to
		// match the properties of this object
	{
		if (!$identity->id)
		{
			$newidentity = $this->createidentity();
			$identity->id = $newidentity->id;
			$identity->createddate = $newidentity->createddate;
			//$identity->lastmodifieddate = $newidentity->lastmodifieddate;
		}

		$identity->lastmodifieddate = TIMENOW;

		$this->hasunverified = false;
		$identity->foreachrelation(array($this, 'getunverifiedstatus'));
		if ($this->hasunverified && $identity->status == 'normal')
			$identity->status = 'requestedverification';
		elseif (!$this->hasunverified && $identity->status == 'requestedverification')
			$identity->status = 'normal';
		unset($this->hasunverified);

		$setclause = implode(',', $this->getsetclauses(array(
			'identity_ID' => $identity->id,
			'identity_birthyear' => $identity->birthyear,
			'identity_comment' => $identity->comment,
			'identity_createddate' => $identity->createddate,
			'identity_lastmodifieddate' => $identity->lastmodifieddate,
			'identity_affiliation' => $identity->affiliation,
			'identity_status' => $identity->status,
		)));

		$result = $this->db->query("
			REPLACE INTO {$this->prefix}identity
			SET $setclause");

		if (!$result) trigger_error("Failure to update identity in database", E_USER_ERROR);

		$this->updateclassifications($identity);
		$this->updateevents($identity);
		$this->updateidentifiers($identity);
		$this->updatenames($identity);
		$this->updateorganisations($identity);
		$this->updatepersons($identity);
		$this->updateresources($identity);

		$controls = &controls::getinstance();
		$indexer = $controls->getcontrol('indexer');
		$indexer->updateindex($identity);

		return true;
	}

	function mergeidentity($oldidentity, $newidentity)
		// merges two identities in the database.  the old one will be deleted
		// and all its information combined with the new one
	{
		// todo not yet implemented
		return false;
	}

	function updateclassifications($identity)
	{
		$headings = array_merge(array(
			'classification_ID',
			'classification_identityID',
			'classification_code',
			'classification_text',
			'classification_type',
			), $this->getcommonheadings('classification_'));

		$keepids = $values = array();
		foreach ($identity->classifications as $el) 
		{
			if ($el->id) $keepids[] = $el->id;

			$values[] = array_merge(array(
				$el->id ? $el->id : null,
				$identity->id,
				$el->code,
				$el->text,
				$el->type,
			), $this->getcommonvalues($el));
		}

		$this->purgerelations($identity->id, 'classification', $keepids);
		$this->updaterelations('classification', $headings, $values);
	}

	function updateevents($identity)
	{
		$headings = array_merge(array(
			'event_ID',
			'event_identityID',
			'event_title',
			'event_type',
			'event_reltype',
			), $this->getcommonheadings('event_'));

		$keepids = $values = array();
		foreach ($identity->events as $el) 
		{
			if ($el->id) $keepids[] = $el->id;

			$values[] = array_merge(array(
				$el->id ? $el->id : null,
				$identity->id,
				$el->title,
				$el->type,
				$el->reltype,
			), $this->getcommonvalues($el));
		}

		$this->purgerelations($identity->id, 'event', $keepids);
		$this->updaterelations('event', $headings, $values);
	}

	function updateidentifiers($identity)
	{
		$headings = array_merge(array(
			'identifier_ID',
			'identifier_identityID',
			'identifier_identifier',
			'identifier_type',
			'identifier_href',
			), $this->getcommonheadings('identifier_'));

		$keepids = $values = array();
		foreach ($identity->identifiers as $el) 
		{
			if ($el->id) $keepids[] = $el->id;

			$values[] = array_merge(array(
				$el->id ? $el->id : null,
				$identity->id,
				$el->identifier,
				$el->type,
				$el->href,
			), $this->getcommonvalues($el));
		}

		$this->purgerelations($identity->id, 'identifier', $keepids);
		$this->updaterelations('identifier', $headings, $values);
	}

	function updatenames($identity)
	{
		$headings = array_merge(array(
			'name_ID',
			'name_identityID',
			'name_surname',
			'name_givennames',
			'name_title',
			'name_surnamefirst',
			'name_type',
			), $this->getcommonheadings('name_'));

		$keepids = $values = array();
		foreach ($identity->names as $el) 
		{
			if ($el->id) $keepids[] = $el->id;

			$values[] = array_merge(array(
				$el->id ? $el->id : null,
				$identity->id,
				$el->surname,
				$el->givennames,
				$el->title,
				$el->surnamefirst,
				$el->type,
			), $this->getcommonvalues($el));
		}

		$this->purgerelations($identity->id, 'name', $keepids);
		$this->updaterelations('name', $headings, $values);
	}

	function updateorganisations($identity)
	{
		$headings = array_merge(array(
			'organisation_ID',
			'organisation_identityID',
			'organisation_title',
			'organisation_type',
			'organisation_reltype',
			), $this->getcommonheadings('organisation_'));

		$keepids = $values = array();
		foreach ($identity->organisations as $el) 
		{
			if ($el->id) $keepids[] = $el->id;

			$values[] = array_merge(array(
				$el->id ? $el->id : null,
				$identity->id,
				$el->title,
				$el->type,
				$el->reltype,
			), $this->getcommonvalues($el));
		}

		$this->purgerelations($identity->id, 'organisation', $keepids);
		$this->updaterelations('organisation', $headings, $values);
	}

	function updatepersons($identity)
	{
		$headings = array_merge(array(
			'person_ID',
			'person_identityID',
			'person_secondidentityID',
			'person_type',
			), $this->getcommonheadings('person_'));

		$keepids = $values = array();

		foreach ($identity->persons as $el) 
		{
			if (isset($el->id)) $keepids[] = $el->id;

			// for persons, there is not any auto-matching/auto-creating facility
			// here, you'll have to take care of that, and ensure related persons
			// have an ID, BEFORE calling this
			if (empty($el->person->id)) trigger_error('Attempt to insert person relation
				which doesn\'t yet have an ID', E_USER_ERROR);

			$values[] = array_merge(array(
				$el->id,
				$identity->id,
				$el->person->id,
				$el->type,
			), $this->getcommonvalues($el));
		}

		$this->purgerelations($identity->id, 'person', $keepids, array('identityID', 'secondidentityID'));
		$this->updaterelations('person', $headings, $values);
	}

	function updateresources($identity)
	{
		$headings = array_merge(array(
			'resource_ID',
			'resource_identityID',
			'resource_title',
			'resource_type',
			'resource_reltype',
			), $this->getcommonheadings('resource_'));

		$keepids = $values = array();
		foreach ($identity->resources as $el) 
		{
			if ($el->id) $keepids[] = $el->id;

			$values[] = array_merge(array(
				$el->id ? $el->id : null,
				$identity->id,
				$el->title,
				$el->type,
				$el->reltype,
			), $this->getcommonvalues($el));
		}

		$this->purgerelations($identity->id, 'resource', $keepids);
		$this->updaterelations('resource', $headings, $values);
	}

	function updatesearchindex($identity)
	{
		$groups = $identity->gettextfieldgroups();

		foreach ($groups as $groupname => $group)
		{
			foreach ($group as $field) if ($field[0] != '' && $field[1] >= 1)
			{
				$text = $field[0];
				$score = $field[1];
				$textfuzzy = new textfuzzy($text);
				/*$phon = $textfuzzy->getphonkeys();
				$phon_rev = $textfuzzy->getphonkeys(true);
				$stem = $textfuzzy->getstemkeys();

				echo $textfuzzy->getnormal() . "\n";
				print_r($phon);
				print_r($phon_rev);
				print_r($stem);
				echo "-------------------------------------------\n";
				 */
			}
		}
	}

	function resolvepeople($identity)
	// resolves people relations in the supplied identity.  for any people relations
	// with no IDs yet, it does an auto match and merge.  for any with IDs, it merges
	// with the existing record.  then it actually makes the required database updates
	// effective.  Therefore this should be done after it is certain this identity
	// will be updated in the database, but before auto-matching
	{
		$controls = &controls::getinstance();
		$fetchidentities = $controls->getcontrol('fetchidentities');

		foreach ($identity->persons as $id => $el)
		{
			// person relation with no id and no names should never happen, but since
			// we're probably past the point of no return we have to wipe it and
			// continue
			if (empty($el->person->id) && empty($el->person->names))
				unset($identity->persons[$id]);

			if (!empty($el->person->persons))
				$this->resolvepeople($el->person);

			if (empty($el->person->id))
			{
				$result = $fetchidentities->matchidentity($el->person);
				$newidentity = $result['newidentity'];
				$newidentity->merge($el->person);
			}
			else
			{
				$identities = $fetchidentities->getidentities(array($el->person->id));
				if (!empty($identities))
				{
					$newidentity = current($identities);
					$newidentity->merge($el->person);
				}
				else
				{
					$newidentity = $el->person;
				}
			}
			$this->updateidentity($newidentity);
			$identity->persons[$id]->person = $newidentity;
		}
	}

	private function purgerelations($identityid, $reltype, $keepids = null, $matchfields = array('identityID'))
	{
		$andnotin = empty($keepids) ? '' 
			: (" AND `{$reltype}_ID` NOT IN (" . implode(',', $keepids) . ")");

		foreach ($matchfields as $matchfield)
		{
			$matchfield = preg_replace('#\W++#', '', $matchfield);
			$this->db->query("DELETE FROM {$this->prefix}$reltype
				WHERE {$reltype}_{$matchfield}=$identityid $andnotin");
		}
	}

	private function updaterelations($reltype, $headings, $values)
	{
		if (empty($values)) return;

		$headingsclause = "(`" . implode('`,`', $headings) . "`)";
		$rows = array();	
		foreach ($values as $rowvals)	
			$rows[] = '(' . implode(',', $this->getsetclauses($rowvals)) . ')';
		$allrows = implode(',', $rows);

		// now update
		$this->db->query("
			REPLACE INTO {$this->prefix}$reltype
			$headingsclause
			VALUES
			$allrows
			");
	}
	
	private function getcommonvalues($el)
		// returns associative array mapping common values to an associative array
	{
		return array(
			$el->startyear,
			$el->endyear,
			$el->comment,
			$el->incidence,
			$el->score,
			$el->verified,
			);
	}

	private function getcommonheadings($prefix)
	{
		return array(
			$prefix.'startyear',
			$prefix.'endyear',
			$prefix.'comment',
			$prefix.'incidence',
			$prefix.'score',
			$prefix.'verified',
		);
	}

	private function getsetclauses($values)
		// safely map an associative array to an array of clauses suitable for 
		// SQL SET clause ie '`column` = "value"'
		// or "value" if the keys aren't strings
	{
		$sets = array();
		foreach ($values as $key => $val)
		{
			if (is_string($val)) $val = "'" . addslashes($val) . "'";
			elseif ($val === null) $val = "NULL";
			elseif (!is_string($val)) $val = (int)$val;
			$sets[] = is_string($key) ? "$key=$val" : $val;
		}
		return $sets;
	}

}

?>
