<?php

/*
	importidentities.inc.php - import multiple identities from an importsource
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/identity.inc.php');
require_once(RESOURCE_DIR . 'includes/importsource.inc.php');

class importidentities
{
	function importidentities(&$db)
	{
		$this->db = &$db;
		$this->prefix = $db->get_prefix();

		$this->progresscallback = null;
	}

	function setprogresscallback($callback)
		// allows you to use a callback function which is called during
		// the import, each time an incoming identity is processed
		// the callback function will be passed a 'status' array
		// consisting of keys such as 'importedcount', 'newcount',
		// 'lastidentityid' and more may be added later.
		// using a callback is optional but since importing any more than
		// one or two identities at once will take some time, it's
		// recommended to hook into this to present a progress indicator
		// to the user
	{
		$this->progresscallback = $callback;
	}

	function setprogress($progress)
	{
		if (is_callable($this->progresscallback))
			call_user_func($this->progresscallback, $progress);
	}

	function import($importsource, $maxidentities = null, $preserveids = false)
		// imports identities.  $importsource should be an object
		// which implements importsource's interface.
		// returns number of records imported, or FALSE on failure
		// $maxidentities allows you to specify a limit - repeated
		// calls with the same importsource will resume from where
		// it left off, allowing you to control import rate or report
		// on progress
		// setting $preserveids to true will preserve id numbers - this
		// should only be done if re-importing identities from the SAME
		// installation; ie restoring from backup
	{
		if (!$importsource->isvalid())
			return false;

		set_time_limit(7200);
		$abort = ignore_user_abort(true);

		static $max_fetched_identities = 16;
		$importedcount = $newcount = 0;
		$status = array(
			'importedcount' => 0,
			'newcount' => 0,
			'lastidentity' => null
		);

		$this->setprogress($status);

		//testing
		//if ($maxidentities === null) $maxidentities = 100;

		$controls = &controls::getinstance();
		$fetchidentities = $controls->getcontrol('fetchidentities');
		$editidentity = $controls->getcontrol('editidentity');

		while (($maxidentities === null || $importedcount	< $maxidentities)
			&& ($identity = $importsource->getnextidentity()) && !connection_aborted())
		{
			if (!$preserveids) $identity->anonymise();
			$identity->localise();

			$editidentity->resolvepeople($identity);

			$result = $fetchidentities->matchidentity($identity);

			$newidentity = $result['newidentity'];

			$editidentity->updateidentity($newidentity);
			$importedcount++;
			if ($result['isnew']) $newcount++;

			$status = array(
				'importedcount' => $importedcount,
				'newcount' => $newcount,
				'lastidentity' => $newidentity,
				);

			$this->setprogress($status);

			//echo "Imported $recordsimported\n";
		}
		
		set_time_limit(30);
		ignore_user_abort($abort);

		return $status;
	}
}

?>
