<?php

/*
	indexer.inc.php - For building and manipulating the search index
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/textfuzzy.inc.php');

class indexer
{
	function indexer(&$db)
	{
		$this->db = &$db;
		$this->prefix = $db->get_prefix();
	}

	function updateindex($identity)
		// removes any existing search index data for this identity and replaces
		// it with the current identity contents 
	{
		$identityid = (int)$identity->id;
		$groups = $identity->gettextfieldgroups();

		$this->deletefromindex($identity);

		$textrows = array();
		$entries = array();
		$stems = array();

		foreach ($groups as $category => $phrases)
		{
			foreach ($phrases as $arr)
			{
				$phrase = $arr[0];
				$score = $arr[1];

				$fuzzy = new textfuzzy((string)$phrase);

				$normal = $fuzzy->getnormal();
				if ($normal != '')
				{
					$normwords = explode(' ', $normal);
					$stemmed = $fuzzy->getstemmed();
					$stemwords = explode(' ', $stemmed);
					$phon = $fuzzy->getphonkeys();
					$phonr = $fuzzy->getphonkeys(true);
					$commonscores = $fuzzy->getcommonwordscores();

					foreach ($normwords as $id => $word)
					{
						$textrows[$word] = "'" . addslashes($word) . "'," . $phon[$id] . "," . $phonr[$id];
						$entries[$word] = array($category, floor($score * $commonscores[$id]));

						if ($stemwords[$id] !== $word)
						{
							$stems[$word] = $stemwords[$id];
							$stemfuzzy = new textfuzzy($stemwords[$id]);
							$stemphon = $stemfuzzy->getphonkeys();
							$stemphonr = $stemfuzzy->getphonkeys(true);
							$textrows[$stemwords[$id]] = "'" . addslashes($stemwords[$id]) . "'," . current($stemphon) . "," . current($stemphonr);
						}
					}
				}
			}
		}

		if (!empty($textrows))
		{
			// insert words, get word ids	
			$textvalues = '(' . implode('),(', $textrows) . ')';
			$this->db->query("INSERT IGNORE INTO {$this->prefix}index_word
				(index_word_text,index_word_phon,index_word_rphon)
				VALUES
				$textvalues
				");
			$wordids = array();
			$wordstext = "'" . implode("','", array_map('addslashes', array_keys($textrows))) . "'";
			$this->db->query("SELECT index_word_ID AS id, index_word_text AS text FROM {$this->prefix}index_word
				WHERE index_word_text IN ($wordstext)");
			while ($row = $this->db->fetch_array())
				$wordids[$row['text']] = $row['id'];
			$this->db->free_result();

			if (!empty($stems))
			{
				// insert stem relations
				$stemrows = array();
				foreach ($stems as $word => $stemword) if (isset($wordids[$word]))
					$stemrows[] = $wordids[$word] . ',' . $wordids[$stemword];
				$stemvalues = '(' . implode('),(', $stemrows) . ')';
				$this->db->query("INSERT IGNORE INTO {$this->prefix}index_stem
					(index_stem_wordID,index_stem_stemwordID)
					VALUES
					$stemvalues
					");
			}

			// populate index
			$indexrows = array();
			foreach ($entries as $word => $entry) if (isset($wordids[$word]))
				$indexrows[] = $wordids[$word] . ',' . $identityid . ",'" . addslashes($entry[0]) . "'," . (int)$entry[1];
			$indexvalues = '(' . implode('),(', $indexrows) . ')';
			$this->db->query("INSERT IGNORE INTO `{$this->prefix}index`
				(index_wordID,index_identityID,index_category,index_score)
				VALUES
				$indexvalues
				");

			// update word frequencies
			$idstext = implode(',', $wordids);
			$this->db->query("UPDATE {$this->prefix}index_word 
				SET index_word_freq=index_word_freq+1
				WHERE index_word_ID IN ($idstext)");
		}
	}

	function deletefromindex($identity)
	{
		$identityid = (int)$identity->id;

		// get list of old words for later cleanup
		$oldwords = array();
		$this->db->query("SELECT index_wordID AS id FROM `{$this->prefix}index`
			WHERE index_identityID=$identityid");
		while ($row = $this->db->fetch_array())
			$oldwords[$row['id']] = true;
		$this->db->free_result();

		// delete old words in index
		$this->db->query("DELETE FROM `{$this->prefix}index`
			WHERE index_identityID=$identityid");

		if (!empty($oldwords))
		{
			$idstext = implode(',', array_keys($oldwords));

			$this->db->query("UPDATE {$this->prefix}index_word SET index_word_freq=index_word_freq-1
				WHERE index_word_ID IN ($idstext)");

			// get redundant words
			$redundant = array();
			$this->db->query("SELECT index_word_ID FROM {$this->prefix}index_word
				LEFT JOIN `{$this->prefix}index` ON index_wordID=index_word_ID
				WHERE index_word_ID IN ($idstext) AND index_wordID IS NULL
				");
			while ($row = $this->db->fetch_array())
				$redundant[] = current($row);
			$this->db->free_result();

			if (!empty($redundant))
			{
				$idstext = implode(',', $redundant);
				$this->db->query("DELETE FROM {$this->prefix}index_word
					WHERE index_word_ID IN ($idstext)");
				$this->db->query("DELETE FROM {$this->prefix}index_stem
					WHERE index_stem_wordID IN ($idstext)");
				$this->db->query("DELETE FROM {$this->prefix}index_stem
					WHERE index_stem_stemwordID IN ($idstext)");
			}
		}

		return $oldwords;
		
	}
}

?>
