<?php

/*
	indexlookup.inc.php - look up identities quickly from the full text index
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/identity.inc.php');
require_once(RESOURCE_DIR . 'includes/textfuzzy.inc.php');

class indexlookup
{
	function indexlookup(&$db)
	{
		$this->db = &$db;
		$this->prefix = $db->get_prefix();

		$this->wordfrequencies = array();
	}

	private function findwordgroups($words, $autocomplete = false, $maxwords = 40)
		// $words is a string of words separated by spaces
		// returns array of groups of word ids - these word ids include exact matches,
		// words in each group are of the form wordid => score where score is 1 to 10
		// from exact match (10) to phoenetically similar (1)
		// stemmed/unstemmed matches, and closest phoenetic matches
		// $autocomplete treats the last word as a prefix, as if
		// the words are still being typed
	{
		$fuzzy = new textfuzzy($words);
		$phonkeys = $fuzzy->getphonkeys();
		$rphonkeys = $fuzzy->getphonkeys(true);
		$normalwords = array_map('addslashes', explode(' ', $fuzzy->getnormal()));
		$stemwords = array_map('addslashes', explode(' ', $fuzzy->getstemmed()));

		if (count($normalwords) > $maxwords)
		{
			$normalwords = array_slice($normalwords, 0, $maxwords);
			$phonkeys = array_slice($phonkeys, 0, $maxwords);
			$rphonkeys = array_slice($rphonkeys, 0, $maxwords);
			$stemwords = array_slice($stemwords, 0, $maxwords);
		}

		$normalmatch = "'" . implode("','", $normalwords) . "'";
		$stemmatch = "'" . implode("','", $stemwords) . "'";
		$phonmatch = implode(',', $phonkeys);

		static 
			$match_norm = 4,
			$match_stem = 3,
			$match_phon = 2,
			$match_near = 1;

		$phonjoins = '';
		$dups = array();
		$phondepth = 6;
		$lookuptable = array();
		foreach ($normalwords as $id => $word)
		{
			if (isset($dups[$word])) continue;
			$dups[$word] = true;
			$phon = $phonkeys[$id];
			$rphon = $phonkeys[$id];
			$stem = $stemwords[$id];
			if ($phondepth)	$phonjoins .= "
					UNION ALL
				(SELECT $id,index_word_text,index_word_ID,$match_near,index_word_freq FROM {$this->prefix}index_word FORCE INDEX(index_word_phon)
					WHERE index_word_phon>$phon ORDER BY index_word_phon LIMIT $phondepth) UNION ALL
				(SELECT $id,index_word_text,index_word_ID,$match_near,index_word_freq FROM {$this->prefix}index_word FORCE INDEX(index_word_phon)
				 	WHERE index_word_phon<$phon ORDER BY index_word_phon DESC LIMIT $phondepth) UNION ALL
				(SELECT $id,index_word_text,index_word_ID,$match_near,index_word_freq FROM {$this->prefix}index_word FORCE INDEX(index_word_rphon)
					WHERE index_word_rphon>$rphon ORDER BY index_word_rphon LIMIT $phondepth) UNION ALL
				(SELECT $id,index_word_text,index_word_ID,$match_near,index_word_freq FROM {$this->prefix}index_word FORCE INDEX(index_word_rphon)
					WHERE index_word_rphon<$rphon ORDER BY index_word_rphon DESC LIMIT $phondepth)
				";

			$lookuptable[$match_norm][$word][] = $id;
			$lookuptable[$match_stem][$stem][] = $id;
			$lookuptable[$match_phon][$phon][] = $id;
		}

		if ($autocomplete)
		{
			$lastword = end($normalwords);
			$phonjoins .= "
					UNION ALL
				(SELECT '$lastword',index_word_text,index_word_ID,$match_norm,index_word_freq
					FROM {$this->prefix}index_word FORCE INDEX(index_word_text)
					WHERE index_word_text LIKE '$lastword%' 
					AND index_word_text <> '$lastword' LIMIT 40)
				";
		}

		$sql = "
			(SELECT index_word_text AS which, index_word_text AS word, index_word_ID AS id, $match_norm AS cat, index_word_freq AS freq
				FROM {$this->prefix}index_word FORCE INDEX (index_word_text) 
				WHERE index_word_text IN ($normalmatch)) UNION ALL
			(SELECT index_word_text, index_word_text, index_word_ID, $match_stem, index_word_freq
				FROM {$this->prefix}index_stem FORCE INDEX (index_stem_stemwordID)
				INNER JOIN {$this->prefix}index_word FORCE INDEX (index_word_text)
		 		ON index_word_ID=index_stem_stemwordID AND index_word_text IN ($stemmatch)) UNION ALL
			(SELECT index_word_phon, index_word_text, index_word_ID, $match_phon, index_word_freq
			 	FROM {$this->prefix}index_word FORCE INDEX (index_word_phon)
				WHERE index_word_phon IN ($phonmatch) LIMIT 60)
			$phonjoins
			";
		$sql = preg_replace("/\s\s++/", ' ', $sql);
		//echo wordwrap($sql);

		$this->db->query($sql);

		$groups = array();
		foreach ($normalwords as $id => $word) $groups[$id] = array();

		static $fuzzystrings = array();
		if (count($fuzzystrings) > 512) $fuzzystrings = array_slice($fuzzystrings, -256);

		while ($row = $this->db->fetch_array())
		{
			$category = $row['cat'];
			$which = $row['which'];

			$groupids = ($category == $match_near ? array((int)$which) : $lookuptable[$category][$which]);

			$checksimilarity = ($category == $match_near || $category == $match_phon);

			foreach ($groupids as $id)
			{
				if ($checksimilarity)
				{
					if (!isset($fuzzystrings[$row['word']])) $fuzzystrings[$row['word']] = new textfuzzy($row['word']);
					if (!isset($fuzzystrings[$normalwords[$id]])) $fuzzystrings[$normalwords[$id]] = new textfuzzy($normalwords[$id]);

					if ($fuzzystrings[$normalwords[$id]]->similarityto($fuzzystrings[$row['word']]) < 0.5)
						continue;
				}

				if (!isset($groups[$id][$row['id']]))
					$groups[$id][$row['id']] = $row['cat'];
			}

			$this->wordfrequencies[$row['id']] = $row['freq'];
		}
		$this->db->free_result();

		return $groups;
	}

	function findmatches($identity)
		// returns array of identity ids representing candidate matches
		// for the given identity
	{
		$textgroups = $identity->gettextfieldgroups();

		$strings = array();

		foreach ($textgroups as $groupname => $group)
			foreach ($group as $id => $value)
		{
			if (isset($strings[$value[0]])) $strings[$value[0]] = max($strings[$value[0]], $value[1]);
			else
				$strings[$value[0]] = $value[1];
		}

		arsort($strings);

		return $this->besteffortmatch(implode(' ', array_keys($strings)));
	}

	private function getscorecode($wordgroups)
		// returns the SQL code necessary for assigning a score to each search
		// result.  this is where all the relevance ranking happens, at least
		// all such ranking that happens BEFORE actually fetching all the
		// returned rows and comparing them more thoroughly.
	{
		$scoreparts = array();
		$joinnum = 0;
		foreach ($wordgroups as $group)
		{
			$wordsbytype = array();
			$frequencybytype = array();
			foreach ($group as $wordid => $score)
			{
				$wordsbytype[$score][$wordid] = true;

				$frequency = isset($this->wordfrequencies[$wordid]) ? $this->wordfrequencies[$wordid] : 0;
				$frequencybytype[$score] = isset($frequencybytype[$score]) ?
					($frequencybytype[$score] + $frequency)
					: $frequency;
			}
			
			if (!empty($wordsbytype))
			{
				$joinnum++;
				foreach ($wordsbytype as $i => $val)
				{
					$wordmatch = count($val) == 1 ? ('='.key($val)) : ('IN (' . implode(',', array_keys($val)) . ')');
					$score = $i == 4 ? "96.0" : ($i == 3 ? "64.0" : ($i == 2 ? "8.0" : "1.0"));
					$score /= (1.0 + log(max($frequency, 0.5), 20));
					$scoreparts[] = "
						COALESCE(
							(SELECT (1.0 + LOG10(COUNT(*))) * $score * (1.0 + LOG10(MAX(s$joinnum.index_score))) 
								FROM `{$this->prefix}index` AS s$joinnum FORCE INDEX(PRIMARY)
								WHERE s$joinnum.index_identityID=id AND s$joinnum.index_wordID $wordmatch
							),
							0)
						";
				}
			}
		}
		$scorecode = empty($scoreparts) ? "NULL"
				: '(' . implode(' + ', $scoreparts) . ')';

		return $scorecode;
	}

	function besteffortmatch($string, $autocomplete = false)
	{
		$wordgroups = $this->findwordgroups($string, $autocomplete, 60);

		if (empty($wordgroups)) return array();
		
		$words = array();
		foreach ($wordgroups as $group)
			foreach ($group as $wordid => $score)
				$words[$wordid] = true;

		$firstjoinwords = array();
		$maxperword = 200;
		$joinnum = 0;
		foreach ($wordgroups as $group)
		{
			$cumfreq = 0;
			foreach ($group as $wordid => $score)
			{
				if (!isset($this->wordfrequencies[$wordid]))
					continue;
				elseif (($cumfreq + $this->wordfrequencies[$wordid]) > $maxperword)
					break;
				else
				{
					$firstjoinwords[$wordid] = true;
					$cumfreq += $this->wordfrequencies[$wordid];
				}
			}
		}
		$dolimit = '';
		$maxtotal = 1000;
		if (empty($firstjoinwords))
		{
			// words are not unique enough, use less accurate first join
			foreach ($wordgroups as $group) if (!empty($group))
			{
				reset($group);
				$firstjoinwords[key($group)] = true;
			}
			$dolimit = "LIMIT $maxtotal"; 
		}
		if (empty($firstjoinwords))
			// no words we can possibly search on, return nothing
			return array();

		$firstjoinmatch = implode(',', array_keys($firstjoinwords));
		$scorecode = $this->getscorecode($wordgroups);
		$sql = "
			SELECT
				id,
				$scorecode AS score
			FROM
				(SELECT DISTINCT i.index_identityID AS id FROM `{$this->prefix}index` AS i
				WHERE i.index_wordID IN ($firstjoinmatch) $dolimit) AS firstjoin
			";
		$sql = preg_replace('/\s\s++/', ' ', $sql);
		//echo wordwrap($sql, 35);

		$this->db->query($sql);
		$scores = array();
		while ($row = $this->db->fetch_array())
			$scores[$row['id']] = $row['score'];

		$this->db->free_result();
		arsort($scores);

		//print_r($scores);

		return array_keys($scores);
	}

	function grammaticalmatch($string, $autocomplete = false)
	{
		// TODO do extra things with grammar like phrase searching, etc
		$wordgroups = $this->findwordgroups($string, $autocomplete, 40);
		if (empty($wordgroups)) return array();
		
		$words = array();
		foreach ($wordgroups as $group)
			foreach ($group as $wordid => $score)
				$words[$wordid] = true;

		// sort by group frequencies so the earlier joins do the most narrowing down
		$groupfrequencies = array();
		foreach ($wordgroups as $groupid => $group)
		{
			$groupfrequencies[$groupid] = 0;
			foreach ($group as $wordid => $type)
			{
				$frequency = isset($this->wordfrequencies[$wordid]) ? $this->wordfrequencies[$wordid] : 0;
				$groupfrequencies[$groupid] += $frequency;
			}
		}

		asort($groupfrequencies);
		if (empty($groupfrequencies) || reset($groupfrequencies) == 0)
			// there are no words, or the rarest word occurs zero times 
			return array();
		
		$grouporder = array_keys($groupfrequencies);

		$firstgroup = array_shift($grouporder);

		$conditions = array();
		foreach ($grouporder as $groupid)
		{
			$group = $wordgroups[$groupid];
			$wordidsmatch = implode(',', array_keys($group));
			$conditions[] = "
				EXISTS (SELECT * FROM `{$this->prefix}index` AS i$groupid
				WHERE i$groupid.index_identityid=id
					AND i$groupid.index_wordid IN ($wordidsmatch))
				";
		}

		$scorecode = $this->getscorecode($wordgroups);
		$wordidsmatch = implode(',', array_keys($wordgroups[$firstgroup]));
		$whereconditionmatch = empty($conditions)
			? ''
			: ('WHERE ' . implode(' AND ', $conditions));
		$dototal = count($wordgroups) > 1 ? 2000 : 1000;
		$sql = "
			SELECT
				id,
				$scorecode AS score
			FROM
				(SELECT DISTINCT li.index_identityID AS id
					FROM `{$this->prefix}index` AS li FORCE INDEX(index_wordID)
					WHERE li.index_wordID IN ($wordidsmatch) LIMIT $dototal) AS firstjoin
			$whereconditionmatch
			";
		$sql = preg_replace('/\s\s++/', ' ', $sql);
		//echo wordwrap($sql, 35);

		$this->db->query($sql);
		$scores = array();
		while ($row = $this->db->fetch_array())
			$scores[$row['id']] = $row['score'];

		$this->db->free_result();
		arsort($scores);

		//print_r($scores);

		return array_keys($scores);
	}
}

?>
