<?php

/*
	classification - a single classification of an identity
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

//if (!defined('RESOURCE_DIR')) define('RESOURCE_DIR', dirname(__FILE__) . '/../');

require_once(RESOURCE_DIR . 'includes/relation.inc.php');
require_once(RESOURCE_DIR . 'includes/probability.inc.php');

class classification extends relation
{
	public $id = null;
	public $code = null;
	public $text = null;
	public $type;
	 
	function getfields()
	{
		return array_merge(array(
			array('type', 'select', 'type', 'Choose the type of classification', null, true, nicnames_config::$resourcetypes['classification_type']),
			array('code', 'string', 'code', 'Does this classification have an alphanumeric code?', 128, false, array('size' => 8)),
			array('text', 'string', 'text label', 'Does this classification have a descriptive label?', 255, false, array('size' => 64)),
			), $this->getcommonfields());
	}

	function getXML($document = null)
	// Create a DomElement object representing this classification
	{
		if ($document===null) $document = new DomDocument();
		$el = $document->createElement('classification');
		if ($this->code != '') $el->appendChild($document->createElement('code', $this->code));
		if ($this->text != '' || $this->code == '')
			$el->appendChild($document->createElement('text', $this->text != '' ? $this->text : ''));
		$el->setAttribute('type', $this->type);
		if ($this->id) $el->setAttribute('id', $this->id);
		$el = $this->getcommonpropertiesXML($el);
		return $el;
	}

	function setfromXML($el)
	// populate this object with properties from the supplied DOM XML element
	{
		if ($el->hasAttribute('id')) $this->id = $el->getAttribute('id');
		
		for ($i = 0, $len = $el->childNodes->length; $i < $len; $i++)
		{
			$node = $el->childNodes->item($i);
			switch ($node->nodeName)
			{
			case 'code':
				$this->code = $node->nodeValue;
				break;
			case 'text':
				$this->text = $node->nodeValue;
				break;
			}
		}

		$this->type = $el->getAttribute('type');

		$this->setcommonfromXML($el);
	}

	function similarityto($other)
	{
		$factors = array();

		$hascode = true;
		if ($this->code != '' && $other->code != '')
		{
			$code1 = new textfuzzy($this->code);
			$code2 = new textfuzzy($other->code);
			$factors['code'] = array($code1->getnormal() == $code2->getnormal(), 140);
		}
		elseif ($this->code != '' || $other->code != '')
			$factors['code'] = array(0, 40);
		else
		{
			$factors['code'] = array(0, 1.5);
			$hascode = false;
		}

		if ($this->text != '' && $other->text != '')
		{
			$text1 = new textfuzzy($this->text);
			$text2 = new textfuzzy($other->text);
			$factors['text'] = array($text1->similarityto($text2), $hascode ? 30 : 140);
		}
		elseif ($this->text != '' || $other->text != '')
			$factors['text'] = array(0, 6);
		else
			$factors['text'] = array(0, 1.5);

		$factors['type'] = array($this->type == $other->type, 4);

		$factors['common'] = $this->commonsimilarityfactor($other);

		return probability::combinedcorrected($factors);
	}

	function getstrings()
		// returns array of strings suitable for human-readable rendering of this
		// piece of informtion.  Contains such fields as 'title', 'subtitle', 
		// 'pre-qualifier', 'post-qualifier', 'comment', etc
	{
		$type = isset(nicnames_config::$resourcetypes['classification_type'][$this->type]) ?
			nicnames_config::$resourcetypes['classification_type'][$this->type] : $this->type;
		return array(
			'title' => $this->code == '' ? null : $this->code,
			'subtitle' => $this->text == '' ? null : $this->text,
			'pre-qualifier' => $type,
		) + $this->getcommonstrings();
	}
}

?>
