<?php

/*
	identifier - a single identifier of an identity
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once(RESOURCE_DIR . 'includes/relation.inc.php');

class identifier extends relation
{
	public $id = null;
	public $identifier = null;
	public $type;
	public $href = null;
	 
	function getfields()
	{
		return array_merge(array(
			array('type', 'select', 'type', 'Choose the type of event', null, true, nicnames_config::$resourcetypes['identifier_type']),
			array('identifier', 'string', 'code', 'Please specify the alphanumeric code of this identifier', 128, true, array('size' => 24)),
			array('href', 'string', 'web address', 'Specify a web address leading to further information, if applicable', 255, false, array('size' => 48)),
			), $this->getcommonfields());
	}

	function getXML($document = null)
	// Create a DomElement object representing this identifier
	{
		if ($document===null) $document = new DomDocument();
		$el = $document->createElement('identifier', $this->identifier != '' ? $this->identifier : '');
		if ($this->href != '') $el->setAttribute('href', $this->href);
		$el->setAttribute('type', $this->type);
		if ($this->id) $el->setAttribute('id', $this->id);
		$el = $this->getcommonpropertiesXML($el);
		return $el;
	}

	function setfromXML($el)
	// populate this object with properties from the supplied DOM XML element
	{
		if ($el->hasAttribute('id')) $this->id = $el->getAttribute('id');
		if ($el->nodeValue != '') $this->identifier = $el->nodeValue;
		$this->type = $el->getAttribute('type');
		$this->reltype = $el->getAttribute('reltype');

		$this->setcommonfromXML($el);
	}

	function similarityto($other)
	{
		$factors = array();

		if ($this->identifier != '' && $other->identifier != '')
		{
			$identifier1 = new textfuzzy($this->identifier);
			$identifier2 = new textfuzzy($other->identifier);
			$factors['identifier'] = array($identifier1->getnormal() == $identifier2->getnormal(), 200);
		}
		elseif ($this->identifier != '' || $other->identifier != '')
			$factors['identifier'] = array(0, 40);
		else
			$factors['identifier'] = array(0, 1.5);

		if ($this->href != '' && $other->href != '')
		{
			$href1 = new textfuzzy($this->href);
			$href2 = new textfuzzy($other->href);
			$factors['href'] = array($href1->getnormal() == $href2->getnormal(), 19);
		}
		elseif ($this->href != '' || $other->href != '')
			$factors['href'] = array(0, 3);

		$factors['type'] = array($this->type == $other->type, 4);

		$factors['common'] = $this->commonsimilarityfactor($other);

		return probability::combinedcorrected($factors);
	}

	function getstrings()
		// returns array of strings suitable for human-readable rendering of this
		// piece of informtion.  Contains such fields as 'title', 'subtitle', 
		// 'pre-qualifier', 'post-qualifier', 'comment', etc
	{
		$type = isset(nicnames_config::$resourcetypes['identifier_type'][$this->type]) ?
			nicnames_config::$resourcetypes['identifier_type'][$this->type] : $this->type;
		return array(
			'title' => $this->identifier == '' ? null : $this->identifier,
			'pre-qualifier' => $type,
			'href' => $this->href == '' ? null : $this->href,
		) + $this->getcommonstrings();
	}

}

?>
