<?php

/*
	probability - helper functions for probabilities, derived from bayes' theorem
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
*/

class probability
{
	static function corrected($p, $weight, $correctionstrength = 10.0)
		// alters the probability value p according to weight - by default, a weight of 10,
		// so something with a weight of 10 will have half the effect, centred around 0.5,
		// and something with weight of 100 will have 10/11ths the effect.
	{
		$num = (0.5 * $correctionstrength) + ($p * $weight);
		$den = (float)$correctionstrength + $weight;
		return $num / $den;
	}

	static function combined($probabilities)
		// given a number of different probabilities array($p) of (result given some fact)
		// calculate the probability of the (result given all facts)
		// this is the combined probabilities calculation detailed in 
		// http://en.wikipedia.org/wiki/Bayesian_spam_filtering
	{
		$inverseprobabilities = array();
		foreach ($probabilities as $p)
			$inverseprobabilities[] = (1.0 - $p);

		$product = array_product($probabilities);
		$invproduct = array_product($inverseprobabilities);

		if ($product + $invproduct == 0) return 0.5;

		return $product / ($product + $invproduct);
	}

	static function combinedcorrected($factors, $correctionstrength = 10.0)
		// $factors must be an array of array($p, $weight)
		// calculates correctedprobability() for each probability, then combines
		// them as in combinedprobability()
		// this saves doing it yourself in a loop
	{
		$probabilities = array();

		foreach ($factors as $val)
			$probabilities[] = self::corrected($val[0], $val[1], $correctionstrength);

		return self::combined($probabilities);
	}
}

?>
