<?php

/*
	sample-config - Sample config.inc.php file for NicNames
	Copyright (c) 2009
	
	This file is part of NicNames.
	
	NicNames is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as 
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.
	
	NicNames is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with NicNames.  If not, see <http://www.gnu.org/licenses/>.
	
*/

/*
	INSTALLATION INSTRUCTIONS
	
	Copy this file to a new file called config.inc.php.  Fill in all the
	information below.
	
*/

// Database server: hostname or ip address of database server.  Typically
// the database is on the same server as the application, so this is
// 'localhost'

define('DB_HOSTNAME', 'localhost');

// Database name: the name of the database on the database server to use.

define('DB_DATABASE', 'nicnames');

// Database username: the MySQL username used to access the nicnames
// database.  Ideally this user should have privileges only for the nicnames
// database, not other databases on the same database server.

define('DB_USERNAME', 'nicnames');

// Database password: the MySQL password for above user.

define('DB_PASSWORD', 'special');

// Database table prefix.  Optional prefix for the start of database tables
// usually this is left blank, and nicnames given its own database.

define('DB_TABLEPREFIX', '');

// Debug mode (true or false).  Debug mode can be helpful for testing
// bugs in an application.  It should, however, NEVER be enabled on a
// live installation, but used only on a private test installation.  Debug
// mode may reveal otherwise secret information such as your MySQL
// login and password.

define('DEBUG', false);

class nicnames_config
{

	// This is where you configure your resource types for each category
	// of relation.  Within each category, types consist of a letter-only
	// name, and a free text description which may include spaces, as shown
	// in the example.  The basic categories (name_type, classification_type
	// etc) cannot be altered, but the entries in each can.

	static $resourcetypes = array(
		// types of names
		'name_type' => array(
			'publishedasname' => 'published as',
			'birthname' => 'birth name',
			'legalname' => 'legal name',
			'employeename' => 'employee name',
			'nickname' => 'nickname',
			'other' => 'other',
			),
		// types of classifications
		'classification_type' => array(
			'fieldofresearch' => 'ANZSRC field of research',
			'lcsh' => 'Library of Congress subject heading',
			'lcc' => 'LCC',
			'other' => 'other',
			),
		// types of events
		'event_type' => array(
			'conference' => 'conference',
			'meeting' => 'meeting',
			'other' => 'other',
			),
		// ways in which a person can be related to an event
		'event_reltype' => array(
			'awardee' => 'award recipient',
			'speaker' => 'presenter',
			'organiser' => 'organiser',
			'other' => 'other',
			),
		// types of identifiers
		// identifiers are intended to uniquely identify a person, for example like a staff
		// number.  classifications are meant to classify a group of people, where multiple
		// people can share the same classification
		'identifier_type' => array(
			'localstaffid' => 'local staff ID',
			'localresearcherid' => 'local researcher ID',
			'emailaddress' => 'email address',
			'openid' => 'OpenID address',
			'website' => 'website address',
			'scopusid' => 'Scopus ID',
			'thomsonresearcherid' => 'Thomson ResearcherID',
			'peopleaustraliapid' => 'People Australia PID',
			'lccn' => 'LCCN',
			'other' => 'other',
			),
		// types of organisations
		'organisation_type' => array(
			'institution' => 'academic institution',
			'company' => 'company',
			'association' => 'association',
			'publisher' => 'publisher',
			'other' => 'other',
			),
		// ways in which a person can be related to an organisation
		'organisation_reltype' => array(
			'employer' => 'employee',
			'publisher' => 'published with',
			'researchcentre' => 'research centre',
			'member' => 'member of',
			'owner' => 'owner of',
			'subscriber' => 'subscribed to',
			'associate' => 'associated with',
			'other' => 'other',
			),
		// ways in which a person can be related to another person, whether another
		// affiliated researcher, or any other person
		'person_type' => array(
			'coauthor' => 'co-author',
			'colleague' => 'colleague',
			'family' => 'family relation',
			'other' => 'other',
			),
		// types of resources or 'things'
		'resource_type' => array(
			'book' => 'book',
			'bookchapter' => 'bookchapter',
			'conferencepaper' => 'conference paper',
			'journal' => 'journal',
			'journalarticle' => 'journal article',
			'technicalreport' => 'technical report',
			'thesis' => 'thesis',
			'newspaperarticle' => 'newspaper article',
			'workingpaper' => 'working paper',
			'essay' => 'essay',
			'speech' => 'speech',
			'conferenceproceedings' => 'conference proceedings',
			'other' => 'other',
			),
		// ways in which a person can be associated with a resource
		'resource_reltype' => array(
			'authored' => 'authored',
			'edited' => 'edited',
			'contributed' => 'contributed to',
			'other' => 'other',
			),
		);

	// private information - which information should NOT be exposed publically
	// that is, should not be exposed in any displayed or exported data in either
	// the web interface or API unless the user is currently logged in or
	// authenticated as a respository administrator

	static $privateinformation = array(
		'name_type' => array('birthname', 'legalname', 'employeename', 'nickname', 'other'),
		'identifier_type' => array('localstaffid', 'localresearcherid', 'emailaddress', 'openid', 'other'),
		'person_type' => array('colleage', 'sibling', 'spouse', 'parentof', 'employerof', 'other'),
		);
	
}
	
?>
