<?php

require_once(dirname(__FILE__) . '/../resources/global.inc.php');
require_once(RESOURCE_DIR . '../tests/libraries/simpletest/autorun.php');

require_once(RESOURCE_DIR . 'libraries/bluestone/db_connection.inc.php');
require_once(RESOURCE_DIR . 'libraries/bluestone/debug.inc.php');
require_once(RESOURCE_DIR . 'includes/controls.inc.php');

// this test is a base test class for anything which tests integration with the database
// note that this will actually set up and then tear down a whole other copy of the
// database with table prefix of "test_" for the purpose of this test

class DatabaseIntegrationTestCase extends UnitTestCase
{
  function __construct($name = null)
	{
		parent::__construct($name);

		$this->db = new db_connection(array(
			'persistent' => false,
			'server' => DB_HOSTNAME,
			'database' => DB_DATABASE,
			'user' => DB_USERNAME,
			'pass' => DB_PASSWORD,
			'prefix' => 'test_' . DB_TABLEPREFIX,
			'charset' => 'utf8',
		));
		$this->prefix = $this->db->get_prefix();
		
		$this->controls = &controls::getinstance();
		$this->controls->setproperties($this->db);

		$this->destroytables();

		$dbinit = $this->controls->getcontrol('dbinit');
		$dbinit->preparetables();
	}

	private function destroytables()
	{
		$this->db->query("SHOW TABLES");
		$tables = array();
		while ($row = $this->db->fetch_array())
			$tables[] = current($row);
		$this->db->free_result();

		foreach ($tables as $table) if (substr($table, 0, strlen($this->prefix)) == $this->prefix)
			$this->db->query("DROP TABLE IF EXISTS `$table`");

		if (is_callable(array("parent","__destruct"))) parent::__destruct();	
	}

	// todo for some reason calling this causes breakages
	/*
	function __destruct()
	{
		$this->destroytables();

		if (is_callable(array("parent","__destruct"))) parent::__destruct();	
	}
	 */

	function testdatabaseinit()
	{
		$this->assertTrue(is_a($this->db, 'db_connection'));
		$this->assertTrue($this->db->is_connected());
		$this->assertTrue($this->db->get_prefix() === $this->prefix);

		$this->assertTrue(is_a($this->controls, 'controls'));
	}

}

?>
