<?php

require_once(dirname(__FILE__) . '/../resources/global.inc.php');
require_once(RESOURCE_DIR . '../tests/libraries/simpletest/autorun.php');

require_once(RESOURCE_DIR . '../tests/dbtest.inc.php');

class IdentityEditTest extends DatabaseIntegrationTestCase
{
	private static $reltypes = array(
		'classification',
		'event',
		'identifier',
		'name',
		'organisation',
		'person',
		'resource'
	);

	function testloadcontrols()
	{
		$this->editidentity = $this->controls->getcontrol('editidentity');
		$this->assertTrue($this->editidentity);
		$this->fetchidentities = $this->controls->getcontrol('fetchidentities');
		$this->assertTrue($this->fetchidentities);
	}

	function testcreateidentity()
	{
		$this->identity = $this->editidentity->createidentity();
		$this->assertTrue($this->identity);
		$this->assertTrue($this->identity->id > 0);
	}

	function testfetchidentity()
	{
		$identities = $this->fetchidentities->getidentities($this->identity->id);
		$this->assertTrue($identities);
		$identity = $identities[$this->identity->id];
		$this->assertTrue($identity->id === $this->identity->id);
		$this->assertTrue($identity->comment === $this->identity->comment);

		// test specify id in array
		$identities = $this->fetchidentities->getidentities(array($this->identity->id));
		$identity2 = $identities[$this->identity->id];
		$this->assertTrue($identity2->id === $this->identity->id);
	}

	function testupdateidentity()
	{
		$classification = new classification();
		$classification->type = "other";
		$classification->code = "somecode";
		$classification->text = "mytext";
		$this->identity->classifications[] = $classification;
		
		$name = new name();
		$name->surname = "Smith";
		$name->givennames = "Turner J.";
		$name->title = "Mr.";
		$name->type = "publishedasname";
		$this->identity->names[] = $name;

		$this->identity->comment = "Comment testing tested documents!";

		$result = $this->editidentity->updateidentity($this->identity);
		$this->assertTrue($result);

		// fetch it back and see if the info stuck
		$identities = $this->fetchidentities->getidentities($this->identity->id);
		$identity = current($identities);
		$this->assertTrue(count($identity->classifications) == 1);
		$this->assertTrue($identity->classifications[0]->code == "somecode");

	}

	function testdeleteidentity()
	{
		$result = $this->editidentity->deleteidentity($this->identity);
		$this->assertTrue($result);

		// make sure it no longer exists
		$identities = $this->fetchidentities->getidentities($this->identity->id);
		$this->assertTrue(empty($identities));

		$result = $this->db->query_single("SELECT * FROM {$this->prefix}identity WHERE identity_ID={$this->identity->id}");
		$this->assertTrue(empty($result));

		foreach (self::$reltypes as $reltype)
		{
			$result = $this->db->query_single("SELECT * FROM {$this->prefix}$reltype WHERE {$reltype}_identityID={$this->identity->id}");
			$this->assertTrue(empty($result));
		}
	}
}

?>
